#include "robber.h"
#include <iostream>
#include <iterator>
#include <cmath>

using namespace std;
using namespace __gnu_cxx;

struct FindCop : SPGoal{
	int _limit;
	const hash_map<string, Player>& _players;
	FindCop(const hash_map<string, Player>& players, int limit = 0) : _players(players), _limit(limit){}
	int operator()(const SPInfo* node) const{
		if (_limit > 0 && node->cost > _limit)
			return -1;
		for(hash_map<string, Player>::const_iterator player = _players.begin();
				player != _players.end(); ++player){
			if (player->second.type != robber && player->second.location == node->name){
				return 1;
			}
		}
		return 0;
	}
};

string Robber::turn(){
	hash_map<string,double> streets;
	Intersection& inter = _intersections[_location];
	for (hash_map<string,StreetType>::const_iterator street = inter.connections.begin();
               street != inter.connections.end(); ++street){
		if (street->second == car){
			cerr << "Discarding: " << street->first << " since car is needed" << endl;
			continue;
		}
		double goodness = 10;
		Intersection& conInter = _intersections[street->first];

		if (conInter.type == bank){
			cerr << "FOUND A BANK" << endl;
			list<string> l = shortestPath(_location, _type, FindCop(_players, 3));
			if (l.size() > 0){
				cerr << "Cop " << l.size() << " intersections away." << endl;
				goodness = 0;
			}else if (_banks[street->first] > 0){
				cerr << "No cop close to bank" << endl;
				return street->first;
			}
		}
		int curx = inter.x;
		int cury = inter.y;

		int newx = conInter.x;
		int newy = conInter.y;
		for (hash_map<string, Player>::const_iterator player = _players.begin();
				player != _players.end(); ++player){
			if (player->first == _name)
				continue;

			int px = _intersections[player->second.location].x;
			int py = _intersections[player->second.location].y;

			double dist1 = sqrt(pow((curx-px),2.0)+pow((cury-py),2.0));
			double dist2 = sqrt(pow((newx-px),2.0)+pow((newy-py),2.0));
			/*cerr << "Original distance: " << dist1 << endl;
			cerr << "New distance: " << dist2 << endl;
			*/


			if (dist2 < 50 && dist2 < dist1)
				goodness *= 0.95;

			if (player->second.type == cop_foot && dist2 <= 3)
				goodness /= 100;
			else if (player->second.type == cop_car && dist2 <= 2)
				goodness /= 100;


		}
		
		cerr << "Street: " << street->first << " goodness: " << goodness << endl;
		streets[street->first] = goodness;
	}
	streets[_oldLocation] /= 10;

	for(hash_map<string,int>::const_iterator bank = _banks.begin();
			bank != _banks.end(); ++bank){
		//cerr << "Handling bank at: " << bank->first << endl;
		if (bank->second > 0){
			list<string> l = shortestPath(_location, _type, SimpleSPGoal(bank->first));
			if (l.size() < 2)
				continue;
			list<string>::iterator i = l.begin();
			if (*++i == "53-and-kimbark")
				cerr << "We should NOT find 53-and-kimbark here" << endl;
			streets[*i] += bank->second/(pow(5.0,double(l.size())));

		}
	}

	/*cerr << "Using route: ";
	copy(v.begin(), v.end(), ostream_iterator<string>(cerr, " : "));
	cerr << endl;
	*/

	string destination;
	double goodness = 0;
	for (hash_map<string,double>::const_iterator dest = streets.begin();
			dest != streets.end(); ++dest){
		cerr << "Goodness: " << dest->second << endl;
		if (dest->second > goodness){
		cerr << "New Goodness: " << dest->second << endl;
			goodness = dest->second;
			destination = dest->first;
		}
	}
	_oldLocation = _location;
		
	return destination;
	
}

int main(){
	Robber robber("robber");
	robber.play();

	return 0;
}
